<?php
/**
 * youtube shortcode
 *
 * Contains shortcode + some improvements over the Embeds syntax @
 * http://codex.wordpress.org/Embeds
 *
 * @example [youtube=http://www.youtube.com/watch?v=wq0rXGLs0YM&amp;fs=1&amp;hl=bg_BG]
 */

// 2008-07-15:

// 01-2011:
// <iframe title="YouTube video player" class="youtube-player" type="text/html" width="640" height="390" src="http://www.youtube.com/embed/Qq9El3ki0_g" frameborder="0" allowFullScreen></iframe>



function youtube_markup( $content ) {
	return preg_replace( '|\[youtube[= ](.+?)]|ie', 'youtube_id("$1")', $content );
}

function youtube_link( $content ) {
	return preg_replace( '!(?:\n|\A)(http://www\.youtube\.com/(v|watch)[^\s]+?)(?:\n|\Z)!ie', '"\n" . youtube_id("$1") . "\n"', $content );
}

/*
 * url can be:
 *    http://www.youtube.com/watch#!v=H2Ncxw1xfck
 *    http://www.youtube.com/watch?v=H2Ncxw1xfck
 *    http://www.youtube.com/watch?v=H2Ncxw1xfck&w=320&h=240&fmt=1&rel=0&showsearch=1&hd=0
 *    http://www.youtube.com/v/jF-kELmmvgA
 *    http://www.youtube.com/v/9FhMMmqzbD8?fs=1&hl=en_US
 */

function get_youtube_id( $url ) {
	$url = trim( $url, ' "' );
	$url = trim( $url );
	$url = str_replace( array( '/v/', '#!v=', '&amp;' ), array( '/?v=', '?v=', '&' ), $url );
	$url = parse_url( $url );

	if ( !isset( $url['query'] ) )
		return false;

	$url['query'] = str_replace( '?', '&', $url['query'] );
	parse_str( $url['query'], $qargs );

	if ( !isset($qargs['v'] ) )
		return false;

	$id = preg_replace( '|[^_a-z0-9-]|i', '', $qargs['v'] );

	return $id;
}

function youtube_id( $url ) {
	if ( !$id = get_youtube_id( $url ) )
		return '<!--YouTube Error: bad URL entered-->';

	$url = str_replace( '&#038;', '&', $url );
	$url = trim( $url, ' "' );
	$url = trim( $url );
	$url = str_replace( array( '/v/', '#!v=', '&amp;' ), array( '/?v=', '?v=', '&' ), $url );
	$url = parse_url( $url );

	if ( !isset( $url['query'] ) )
		return false;

	$url['query'] = str_replace( '?', '&', $url['query'] );
	parse_str( $url['query'], $qargs );

	$agent = $_SERVER['HTTP_USER_AGENT'];
	// Bloglines & Google Reader handle YouTube well now, instead of
	// big blank space of yester year, so they can skip this treatment
	if ( is_feed() && !preg_match( '#Bloglines|FeedFetcher-Google#i', $agent ) )
		return '<span style="text-align:center; display: block;"><a href="' . get_permalink() . '"><img src="http://img.youtube.com/vi/' . $id . '/2.jpg" alt="" /></a></span>';

	// calculate the width and height, taken content_width into consideration
	global $content_width;

	$input_w = ( isset($qargs['w'] ) && intval( $qargs['w'] ) ) ? intval( $qargs['w'] ) : 0;
	$input_h = ( isset($qargs['h'] ) && intval( $qargs['h'] ) ) ? intval( $qargs['h'] ) : 0;

	$default_width = 640;

	if ( $input_w > 0 && $input_h > 0 ) {
		$w = $input_w;
		$h = $input_h;
	} elseif ( 0 == $input_w && 0 == $input_h ) {
		if ( isset( $qargs['fmt'] ) && intval( $qargs['fmt'] ) )
			$w = ( !empty( $content_width ) ? min( $content_width, 480 ) : 480 );
		else
			$w = ( !empty( $content_width ) ? min( $content_width, $default_width ) : $default_width );

		$h = ceil( ( $w / 16 ) * 9 ) + 30;
	} elseif ( $input_w > 0 ) {
		$w = $input_w;
		$h = ceil( ( $w / 16 ) * 9 ) + 30;
	} else {
		if ( isset( $qargs['fmt'] ) && intval( $qargs['fmt'] ) )
			$w = ( !empty( $content_width ) ? min( $content_width, 480) : 480 );
		else
			$w = ( !empty( $content_width ) ? min( $content_width, $default_width ) : $default_width );

		$h = $input_h;
	}

	$w = (int) apply_filters( 'youtube_width', $w );
	$h = (int) apply_filters( 'youtube_height', $h );

	$fmt = '';
	if ( isset( $qargs['fmt'] ) && intval( $qargs['fmt'] ) )
		$fmt = '&fmt=' . (int) $qargs['fmt'];

	if ( isset( $qargs['rel'] ) && 0 == $qargs['rel'] )
		$rel = 0;
	else
		$rel = 1;

	if ( isset( $qargs['showsearch'] ) && 1 == $qargs['showsearch'] )
		$search = 1;
	else
		$search = 0;

	if ( isset( $qargs['showinfo'] ) && 0 == $qargs['showinfo'] )
		$info = 0;
	else
		$info = 1;

	if ( isset( $qargs['iv_load_policy'] ) && 3 == $qargs['iv_load_policy'] )
		$iv = 3;
	else
		$iv = 1;

	$start = '';
	if ( isset( $qargs['start'] ) && intval( $qargs['start'] ) )
		$start = '&start=' . (int) $qargs['start'];

	$hd = '';
	if ( isset( $qargs['hd'] ) && intval( $qargs['hd'] ) )
		$hd = '&hd=' . (int) $qargs['hd'];

	return "<span style='text-align:center; display: block;'><object width='$w' height='$h'><param name='movie' value='http://www.youtube.com/v/$id?version=3&rel=$rel&fs=1$fmt&showsearch=$search&showinfo=$info&iv_load_policy=$iv$start$hd' /> <param name='allowfullscreen' value='true' /> <param name='wmode' value='opaque' /> <embed src='http://www.youtube.com/v/$id?version=3&rel=$rel&fs=1$fmt&showsearch=$search&showinfo=$info&iv_load_policy=$iv$start$hd' type='application/x-shockwave-flash' allowfullscreen='true' width='$w' height='$h' wmode='opaque'></embed> </object></span>";
}

function youtube_shortcode( $atts ) {
	if ( isset ( $atts[0] ) )
		$src = ltrim( $atts[0] , '=' );
	else
		$src = shortcode_new_to_old_params( $atts );

	return youtube_id( $src );
}
add_shortcode( 'youtube', 'youtube_shortcode' );
